from __future__ import annotations

from .BitextMiningEvaluator import *
from .ClassificationEvaluator import *
from .ClusteringEvaluator import *
from .Image.Any2AnyMultiChoiceEvaluator import *
from .Image.Any2AnyRetrievalEvaluator import *
from .Image.ClassificationEvaluator import *
from .Image.ClusteringEvaluator import *
from .Image.ImageTextPairClassificationEvaluator import *
from .Image.VisualSTSEvaluator import *
from .Image.ZeroShotClassificationEvaluator import *
from .PairClassificationEvaluator import *
from .RerankingEvaluator import *
from .RetrievalEvaluator import *
from .STSEvaluator import *
from .SummarizationEvaluator import *
