from __future__ import annotations

from .eng.BLINKIT2IRetrieval import *
from .eng.BLINKIT2TRetrieval import *
from .eng.CIRRIT2IRetrieval import *
from .eng.CUB200I2IRetrieval import *
from .eng.EDIST2ITRetrieval import *
from .eng.EncyclopediaVQAIT2ITRetrieval import *
from .eng.Fashion200kI2TRetrieval import *
from .eng.Fashion200kT2IRetrieval import *
from .eng.FashionIQIT2IRetrieval import *
from .eng.Flickr30kI2TRetrieval import *
from .eng.Flickr30kT2IRetrieval import *
from .eng.FORBI2IRetrieval import *
from .eng.GLDv2I2IRetrieval import *
from .eng.GLDv2I2TRetrieval import *
from .eng.HatefulMemesI2TRetrieval import *
from .eng.HatefulMemesT2IRetrieval import *
from .eng.ImageCoDeT2IRetrieval import *
from .eng.InfoSeekIT2ITRetrieval import *
from .eng.InfoSeekIT2TRetrieval import *
from .eng.LLaVAIT2TRetrieval import *
from .eng.MemotionI2TRetrieval import *
from .eng.MemotionT2IRetrieval import *
from .eng.METI2IRetrieval import *
from .eng.MSCOCOI2TRetrieval import *
from .eng.MSCOCOT2IRetrieval import *
from .eng.NIGHTSI2IRetrieval import *
from .eng.OKVQAIT2TRetrieval import *
from .eng.OVENIT2ITRetrieval import *
from .eng.OVENIT2TRetrieval import *
from .eng.ReMuQIT2TRetrieval import *
from .eng.ROxfordI2IRetrieval import *
from .eng.RP2kI2IRetrieval import *
from .eng.RParisI2IRetrieval import *
from .eng.SciMMIRI2TRetrieval import *
from .eng.SciMMIRT2IRetrieval import *
from .eng.SketchyI2IRetrieval import *
from .eng.SOPI2IRetrieval import *
from .eng.StanfordCarsI2IRetrieval import *
from .eng.TUBerlinT2IRetrieval import *
from .eng.VidoreBenchRetrieval import *
from .eng.VisualNewsI2TRetrieval import *
from .eng.VisualNewsT2IRetrieval import *
from .eng.VizWizIT2TRetrieval import *
from .eng.VQA2IT2TRetrieval import *
from .eng.WebQAT2ITRetrieval import *
from .eng.WebQAT2TRetrieval import *
from .multilingual.VdrMultilingualRetrieval import *
from .multilingual.WITT2IRetrieval import *
from .multilingual.XFlickr30kCoT2IRetrieval import *
from .multilingual.XM3600T2IRetrieval import *
